// src/components.ts
import * as t from "@babel/types";

// src/core/get-import-identifier.ts
import { addNamed } from "@babel/helper-module-imports";
function getImportIdentifier(state, path, name, source) {
  const id = `${source}[${name}]`;
  const current = state.hooks.get(id);
  if (current) {
    return current;
  }
  const newID = addNamed(path, name, source);
  state.hooks.set(id, newID);
  return newID;
}

// src/components.ts
var COMPONENTS = {
  // Components
  For: ["For", "solid-js"],
  Show: ["Show", "solid-js"],
  Switch: ["Switch", "solid-js"],
  Match: ["Match", "solid-js"],
  Index: ["Index", "solid-js"],
  ErrorBoundary: ["ErrorBoundary", "solid-js"],
  Suspense: ["Suspense", "solid-js"],
  SuspenseList: ["SuspenseList", "solid-js"],
  Dynamic: ["Dynamic", "solid-js/web"],
  Portal: ["Portal", "solid-js/web"],
  Assets: ["Assets", "solid-js/web"],
  HydrationScript: ["HydrationScript", "solid-js/web"],
  NoHydration: ["NoHydration", "solid-js/web"]
};
var NAMESPACE = "solid";
var NAMESPACE_COMPONENTS = {
  // Components
  for: ["For", "solid-js"],
  show: ["Show", "solid-js"],
  switch: ["Switch", "solid-js"],
  match: ["Match", "solid-js"],
  index: ["Index", "solid-js"],
  "error-boundary": ["ErrorBoundary", "solid-js"],
  suspense: ["Suspense", "solid-js"],
  "suspense-list": ["SuspenseList", "solid-js"],
  dynamic: ["Dynamic", "solid-js/web"],
  portal: ["Portal", "solid-js/web"],
  assets: ["Assets", "solid-js/web"],
  "hydration-script": ["HydrationScript", "solid-js/web"],
  "no-hydration": ["NoHydration", "solid-js/web"]
};
function transformComponents(state, path) {
  path.traverse({
    Expression(p) {
      if (t.isIdentifier(p.node) && !p.scope.hasBinding(p.node.name) && p.node.name in COMPONENTS) {
        const [name, source] = COMPONENTS[p.node.name];
        p.replaceWith(getImportIdentifier(state, p, name, source));
      }
    },
    JSXElement(p) {
      const { openingElement, closingElement } = p.node;
      const id = openingElement.name;
      let replacement;
      if (t.isJSXNamespacedName(id) && id.namespace.name === NAMESPACE && id.name.name in NAMESPACE_COMPONENTS) {
        const [name, source] = NAMESPACE_COMPONENTS[id.name.name];
        const identifier8 = getImportIdentifier(state, p, name, source);
        replacement = t.jsxIdentifier(identifier8.name);
      }
      if (t.isJSXIdentifier(id) && !p.scope.hasBinding(id.name) && id.name in COMPONENTS) {
        const [name, source] = COMPONENTS[id.name];
        const identifier8 = getImportIdentifier(state, p, name, source);
        replacement = t.jsxIdentifier(identifier8.name);
      }
      if (replacement) {
        openingElement.name = replacement;
        if (closingElement) {
          closingElement.name = replacement;
        }
      }
    }
  });
}

// src/transform-ctf.ts
import * as t14 from "@babel/types";

// src/core/deref-signal-variable.ts
import * as t5 from "@babel/types";

// src/core/deref-signal.ts
import * as t4 from "@babel/types";

// src/core/assert.ts
function assert(cond, error) {
  if (!cond) {
    throw error;
  }
}

// src/core/errors.ts
function unexpectedType(path, received, expected) {
  return path.buildCodeFrameError(`Unexpected '${received}' (Expected: ${expected})`);
}
function unexpectedArgumentLength(path, received, expected) {
  return path.buildCodeFrameError(`Unexpected argument length of ${received} (Expected: ${expected})`);
}

// src/core/is-awaited.ts
import * as t2 from "@babel/types";
function isAwaited(node) {
  if (t2.isAwaitExpression(node)) {
    return true;
  }
  if (t2.isTemplateLiteral(node)) {
    return node.expressions.some((expr) => t2.isExpression(expr) && isAwaited(expr));
  }
  if (t2.isLiteral(node) || t2.isIdentifier(node) || t2.isArrowFunctionExpression(node) || t2.isFunctionExpression(node) || t2.isClassExpression(node) || t2.isYieldExpression(node) || t2.isJSX(node) || t2.isMetaProperty(node) || t2.isSuper(node) || t2.isThisExpression(node) || t2.isImport(node) || t2.isDoExpression(node)) {
    return false;
  }
  if (t2.isTaggedTemplateExpression(node)) {
    return isAwaited(node.tag) || isAwaited(node.quasi);
  }
  if (t2.isUnaryExpression(node) || t2.isUpdateExpression(node) || t2.isSpreadElement(node)) {
    return isAwaited(node.argument);
  }
  if (t2.isParenthesizedExpression(node) || t2.isTypeCastExpression(node) || t2.isTSAsExpression(node) || t2.isTSSatisfiesExpression(node) || t2.isTSNonNullExpression(node) || t2.isTSTypeAssertion(node) || t2.isTSInstantiationExpression(node)) {
    return isAwaited(node.expression);
  }
  if (t2.isArrayExpression(node) || t2.isTupleExpression(node)) {
    return node.elements.some((el) => el != null && isAwaited(el));
  }
  if (t2.isAssignmentExpression(node)) {
    if (isAwaited(node.right)) {
      return true;
    }
    if (t2.isExpression(node.left)) {
      return isAwaited(node.left);
    }
    return false;
  }
  if (t2.isBinaryExpression(node)) {
    if (t2.isExpression(node.left) && isAwaited(node.left)) {
      return true;
    }
    return isAwaited(node.right);
  }
  if (t2.isCallExpression(node) || t2.isOptionalCallExpression(node) || t2.isNewExpression(node)) {
    if (t2.isExpression(node.callee) && isAwaited(node.callee)) {
      return true;
    }
    return node.arguments.some((arg) => arg && (t2.isSpreadElement(arg) || t2.isExpression(arg)) && isAwaited(arg));
  }
  if (t2.isConditionalExpression(node)) {
    return isAwaited(node.test) || isAwaited(node.consequent) || isAwaited(node.alternate);
  }
  if (t2.isLogicalExpression(node)) {
    return isAwaited(node.left) || isAwaited(node.right);
  }
  if (t2.isMemberExpression(node) || t2.isOptionalMemberExpression(node)) {
    return isAwaited(node.object) || node.computed && t2.isExpression(node.property) && isAwaited(node.property);
  }
  if (t2.isSequenceExpression(node)) {
    return node.expressions.some(isAwaited);
  }
  if (t2.isObjectExpression(node) || t2.isRecordExpression(node)) {
    return node.properties.some((prop) => {
      if (t2.isObjectProperty(prop)) {
        if (t2.isExpression(prop.value) && isAwaited(prop.value)) {
          return true;
        }
        if (prop.computed && t2.isExpression(prop.key) && isAwaited(prop.key)) {
          return true;
        }
        return false;
      }
      if (t2.isSpreadElement(prop)) {
        return isAwaited(prop);
      }
      return false;
    });
  }
  if (t2.isBindExpression(node)) {
    return isAwaited(node.object) || isAwaited(node.callee);
  }
  return false;
}

// src/core/is-yielded.ts
import * as t3 from "@babel/types";
function isYielded(node) {
  if (t3.isYieldExpression(node)) {
    return true;
  }
  if (t3.isTemplateLiteral(node)) {
    return node.expressions.some((expr) => t3.isExpression(expr) && isYielded(expr));
  }
  if (t3.isLiteral(node) || t3.isIdentifier(node) || t3.isArrowFunctionExpression(node) || t3.isFunctionExpression(node) || t3.isClassExpression(node) || t3.isYieldExpression(node) || t3.isJSX(node) || t3.isMetaProperty(node) || t3.isSuper(node) || t3.isThisExpression(node) || t3.isImport(node) || t3.isDoExpression(node)) {
    return false;
  }
  if (t3.isTaggedTemplateExpression(node)) {
    return isYielded(node.tag) || isYielded(node.quasi);
  }
  if (t3.isUnaryExpression(node) || t3.isUpdateExpression(node) || t3.isSpreadElement(node)) {
    return isYielded(node.argument);
  }
  if (t3.isParenthesizedExpression(node) || t3.isTypeCastExpression(node) || t3.isTSAsExpression(node) || t3.isTSSatisfiesExpression(node) || t3.isTSNonNullExpression(node) || t3.isTSTypeAssertion(node) || t3.isTSInstantiationExpression(node)) {
    return isYielded(node.expression);
  }
  if (t3.isArrayExpression(node) || t3.isTupleExpression(node)) {
    return node.elements.some((el) => el != null && isYielded(el));
  }
  if (t3.isAssignmentExpression(node)) {
    if (isYielded(node.right)) {
      return true;
    }
    if (t3.isExpression(node.left)) {
      return isYielded(node.left);
    }
    return false;
  }
  if (t3.isBinaryExpression(node)) {
    if (t3.isExpression(node.left) && isYielded(node.left)) {
      return true;
    }
    return isYielded(node.right);
  }
  if (t3.isCallExpression(node) || t3.isOptionalCallExpression(node) || t3.isNewExpression(node)) {
    if (t3.isExpression(node.callee) && isYielded(node.callee)) {
      return true;
    }
    return node.arguments.some((arg) => arg && (t3.isSpreadElement(arg) || t3.isExpression(arg)) && isYielded(arg));
  }
  if (t3.isConditionalExpression(node)) {
    return isYielded(node.test) || isYielded(node.consequent) || isYielded(node.alternate);
  }
  if (t3.isLogicalExpression(node)) {
    return isYielded(node.left) || isYielded(node.right);
  }
  if (t3.isMemberExpression(node) || t3.isOptionalMemberExpression(node)) {
    return isYielded(node.object) || node.computed && t3.isExpression(node.property) && isYielded(node.property);
  }
  if (t3.isSequenceExpression(node)) {
    return node.expressions.some(isYielded);
  }
  if (t3.isObjectExpression(node) || t3.isRecordExpression(node)) {
    return node.properties.some((prop) => {
      if (t3.isObjectProperty(prop)) {
        if (t3.isExpression(prop.value) && isYielded(prop.value)) {
          return true;
        }
        if (prop.computed && t3.isExpression(prop.key) && isYielded(prop.key)) {
          return true;
        }
        return false;
      }
      if (t3.isSpreadElement(prop)) {
        return isYielded(prop);
      }
      return false;
    });
  }
  if (t3.isBindExpression(node)) {
    return isYielded(node.object) || isYielded(node.callee);
  }
  return false;
}

// src/core/unwrap-node.ts
function isNestedExpression(node) {
  switch (node.type) {
    case "ParenthesizedExpression":
    case "TypeCastExpression":
    case "TSAsExpression":
    case "TSSatisfiesExpression":
    case "TSNonNullExpression":
    case "TSTypeAssertion":
    case "TSInstantiationExpression":
      return true;
    default:
      return false;
  }
}
function unwrapNode(node, key) {
  if (key(node)) {
    return node;
  }
  if (isNestedExpression(node)) {
    return unwrapNode(node.expression, key);
  }
  return void 0;
}

// src/core/deref-signal.ts
var REF_SIGNAL_CTF = "$refSignal";
var GET_CTF = "$get";
var SET_CTF = "$set";
var CALL_CTF = /* @__PURE__ */ new Set([REF_SIGNAL_CTF, GET_CTF, SET_CTF]);
var GETTER_CTF = "$getter";
var SETTER_CTF = "$setter";
var PROPERTY_CTF = "$property";
var OBJECT_PROPERTY_CTF = /* @__PURE__ */ new Set([GETTER_CTF, SETTER_CTF, PROPERTY_CTF]);
function derefSignal(path, signalIdentifier, readIdentifier, writeIdentifier) {
  path.scope.path.traverse({
    CallExpression(p) {
      if (p.scope !== path.scope && p.scope.hasOwnBinding(signalIdentifier.name)) {
        return;
      }
      const trueCallee = unwrapNode(p.node.callee, t4.isIdentifier);
      if (!trueCallee || p.scope.hasBinding(trueCallee.name) || !CALL_CTF.has(trueCallee.name)) {
        return;
      }
      const rawArgs = p.node.arguments[0];
      const arg = unwrapNode(rawArgs, t4.isIdentifier);
      assert(arg, unexpectedType(p, rawArgs.type, "Identifier"));
      if (arg.name !== signalIdentifier.name) {
        return;
      }
      if (trueCallee.name === REF_SIGNAL_CTF) {
        p.replaceWith(
          t4.arrayExpression([
            readIdentifier,
            writeIdentifier
          ])
        );
      }
      if (trueCallee.name === GET_CTF) {
        p.replaceWith(readIdentifier);
      }
      if (trueCallee.name === SET_CTF) {
        p.replaceWith(writeIdentifier);
      }
    },
    ObjectProperty(p) {
      if (p.scope !== path.scope && p.scope.hasOwnBinding(signalIdentifier.name)) {
        return;
      }
      if (p.node.shorthand && !p.node.computed) {
        if (t4.isIdentifier(p.node.key) && t4.isIdentifier(p.node.value) && p.node.key.name === signalIdentifier.name && p.node.value.name === signalIdentifier.name) {
          p.replaceWith(
            t4.objectProperty(
              p.node.key,
              t4.callExpression(readIdentifier, [])
            )
          );
        }
        return;
      }
      const trueCallExpr = unwrapNode(p.node.value, t4.isCallExpression);
      if (trueCallExpr) {
        const trueCallee = unwrapNode(trueCallExpr.callee, t4.isIdentifier);
        if (!trueCallee || p.scope.hasBinding(trueCallee.name) || !OBJECT_PROPERTY_CTF.has(trueCallee.name)) {
          return;
        }
        assert(!t4.isPrivateName(p.node.key), unexpectedType(p, "PrivateName", "Expression"));
        const arg = trueCallExpr.arguments[0];
        assert(t4.isIdentifier(arg), unexpectedType(p, arg.type, "Identifier"));
        if (arg.name !== signalIdentifier.name) {
          return;
        }
        if (trueCallee.name === SETTER_CTF) {
          const param = p.scope.generateUidIdentifier("value");
          p.replaceWith(
            t4.objectMethod(
              "set",
              p.node.key,
              [param],
              t4.blockStatement([
                t4.returnStatement(
                  t4.callExpression(
                    writeIdentifier,
                    [
                      t4.arrowFunctionExpression(
                        [],
                        param
                      )
                    ]
                  )
                )
              ])
            )
          );
        }
        if (trueCallee.name === GETTER_CTF) {
          p.replaceWith(
            t4.objectMethod(
              "get",
              p.node.key,
              [],
              t4.blockStatement([
                t4.returnStatement(
                  t4.callExpression(
                    readIdentifier,
                    []
                  )
                )
              ])
            )
          );
        }
        if (trueCallee.name === PROPERTY_CTF) {
          const param = p.scope.generateUidIdentifier("value");
          p.replaceWithMultiple([
            t4.objectMethod(
              "get",
              p.node.key,
              [],
              t4.blockStatement([
                t4.returnStatement(
                  t4.callExpression(
                    readIdentifier,
                    []
                  )
                )
              ])
            ),
            t4.objectMethod(
              "set",
              p.node.key,
              [param],
              t4.blockStatement([
                t4.returnStatement(
                  t4.callExpression(
                    writeIdentifier,
                    [
                      t4.arrowFunctionExpression(
                        [],
                        param
                      )
                    ]
                  )
                )
              ])
            )
          ]);
        }
      }
    },
    Expression(p) {
      if (p.scope !== path.scope && p.scope.hasOwnBinding(signalIdentifier.name)) {
        return;
      }
      if (t4.isIdentifier(p.node) && p.node.name === signalIdentifier.name) {
        p.replaceWith(t4.callExpression(readIdentifier, []));
      }
    },
    UpdateExpression(p) {
      if (p.scope !== path.scope && p.scope.hasOwnBinding(signalIdentifier.name)) {
        return;
      }
      if (t4.isIdentifier(p.node.argument) && p.node.argument.name === signalIdentifier.name) {
        const param = p.scope.generateUidIdentifier("current");
        if (p.node.prefix) {
          p.replaceWith(
            t4.callExpression(
              writeIdentifier,
              [
                t4.arrowFunctionExpression(
                  [param],
                  t4.binaryExpression(
                    p.node.operator === "++" ? "+" : "-",
                    param,
                    t4.numericLiteral(1)
                  )
                )
              ]
            )
          );
        } else {
          p.replaceWith(
            t4.callExpression(
              t4.arrowFunctionExpression(
                [],
                t4.blockStatement([
                  t4.variableDeclaration(
                    "const",
                    [
                      t4.variableDeclarator(
                        param,
                        t4.callExpression(
                          readIdentifier,
                          []
                        )
                      )
                    ]
                  ),
                  t4.expressionStatement(
                    t4.callExpression(
                      writeIdentifier,
                      [
                        t4.arrowFunctionExpression(
                          [],
                          t4.binaryExpression(
                            p.node.operator === "++" ? "+" : "-",
                            param,
                            t4.numericLiteral(1)
                          )
                        )
                      ]
                    )
                  ),
                  t4.returnStatement(
                    param
                  )
                ])
              ),
              []
            )
          );
        }
      }
    },
    AssignmentExpression(p) {
      if (p.scope !== path.scope && p.scope.hasOwnBinding(signalIdentifier.name)) {
        return;
      }
      const identifier8 = p.node.left;
      let expression = p.node.right;
      if (isAwaited(expression) || isYielded(expression)) {
        const statement = p.getStatementParent();
        const functionParent = p.getFunctionParent();
        if (statement) {
          const awaitedID = statement.scope.generateUidIdentifier("tmp");
          const declaration = t4.variableDeclaration(
            "const",
            [t4.variableDeclarator(
              awaitedID,
              expression
            )]
          );
          if (functionParent) {
            if (functionParent.isAncestor(statement)) {
              statement.insertBefore(declaration);
            } else {
              functionParent.scope.push({
                id: awaitedID,
                init: expression,
                kind: "const"
              });
            }
          } else {
            statement.insertBefore(declaration);
          }
          expression = awaitedID;
        }
      }
      if (t4.isIdentifier(identifier8) && identifier8.name === signalIdentifier.name) {
        let arg;
        if (p.node.operator === "=") {
          arg = t4.arrowFunctionExpression([], expression);
        } else {
          const param = p.scope.generateUidIdentifier("current");
          arg = t4.arrowFunctionExpression(
            [param],
            t4.assignmentExpression(
              p.node.operator,
              param,
              expression
            )
          );
        }
        p.replaceWith(t4.callExpression(writeIdentifier, [arg]));
      }
    }
  });
}

// src/core/deref-signal-variable.ts
function derefSignalVariable(path, signalIdentifier, stateIdentifier) {
  const readIdentifier = path.scope.generateUidIdentifier(signalIdentifier.name);
  const writeIdentifier = path.scope.generateUidIdentifier(`set${signalIdentifier.name}`);
  derefSignal(
    path,
    signalIdentifier,
    readIdentifier,
    writeIdentifier
  );
  return t5.variableDeclarator(
    t5.arrayPattern([
      readIdentifier,
      writeIdentifier
    ]),
    stateIdentifier
  );
}

// src/core/accessor-variable.ts
import * as t7 from "@babel/types";

// src/core/deref-memo.ts
import * as t6 from "@babel/types";
var REF_MEMO_CTF = "$refMemo";
var GET_CTF2 = "$get";
var CALL_CTF2 = /* @__PURE__ */ new Set([REF_MEMO_CTF, GET_CTF2]);
var GETTER_CTF2 = "$getter";
var PROPERTY_CTF2 = "$property";
var OBJECT_PROPERTY_CTF2 = /* @__PURE__ */ new Set([GETTER_CTF2, PROPERTY_CTF2]);
function derefMemo(path, memoIdentifier, readIdentifier) {
  path.scope.path.traverse({
    CallExpression(p) {
      if (p.scope !== path.scope && p.scope.hasOwnBinding(memoIdentifier.name)) {
        return;
      }
      const trueCallee = unwrapNode(p.node.callee, t6.isIdentifier);
      if (!trueCallee || p.scope.hasBinding(trueCallee.name) || !CALL_CTF2.has(trueCallee.name)) {
        return;
      }
      const rawArgs = p.node.arguments[0];
      const arg = unwrapNode(rawArgs, t6.isIdentifier);
      assert(arg, unexpectedType(p, rawArgs.type, "Identifier"));
      if (arg.name !== memoIdentifier.name) {
        return;
      }
      if (trueCallee.name === REF_MEMO_CTF) {
        p.replaceWith(readIdentifier);
      }
      if (trueCallee.name === GET_CTF2) {
        p.replaceWith(readIdentifier);
      }
    },
    ObjectProperty(p) {
      if (p.scope !== path.scope && p.scope.hasOwnBinding(memoIdentifier.name)) {
        return;
      }
      if (p.node.shorthand && !p.node.computed) {
        if (t6.isIdentifier(p.node.key) && t6.isIdentifier(p.node.value) && p.node.key.name === memoIdentifier.name && p.node.value.name === memoIdentifier.name) {
          p.replaceWith(
            t6.objectProperty(
              p.node.key,
              t6.callExpression(readIdentifier, [])
            )
          );
        }
        return;
      }
      const trueCallExpr = unwrapNode(p.node.value, t6.isCallExpression);
      if (trueCallExpr) {
        const trueCallee = unwrapNode(trueCallExpr.callee, t6.isIdentifier);
        if (!trueCallee || p.scope.hasBinding(trueCallee.name) || !OBJECT_PROPERTY_CTF2.has(trueCallee.name)) {
          return;
        }
        assert(!t6.isPrivateName(p.node.key), unexpectedType(p, "PrivateName", "Expression"));
        const arg = trueCallExpr.arguments[0];
        assert(t6.isIdentifier(arg), unexpectedType(p, arg.type, "Identifier"));
        if (arg.name !== memoIdentifier.name) {
          return;
        }
        if (trueCallee.name === GETTER_CTF2) {
          p.replaceWith(
            t6.objectMethod(
              "get",
              p.node.key,
              [],
              t6.blockStatement([
                t6.returnStatement(
                  t6.callExpression(
                    readIdentifier,
                    []
                  )
                )
              ])
            )
          );
        }
        if (trueCallee.name === PROPERTY_CTF2) {
          p.replaceWith(
            t6.objectMethod(
              "get",
              p.node.key,
              [],
              t6.blockStatement([
                t6.returnStatement(
                  t6.callExpression(
                    readIdentifier,
                    []
                  )
                )
              ])
            )
          );
        }
      }
    },
    Expression(p) {
      if (p.scope !== path.scope && p.scope.hasOwnBinding(memoIdentifier.name)) {
        return;
      }
      if (t6.isIdentifier(p.node) && p.node.name === memoIdentifier.name) {
        p.replaceWith(t6.callExpression(readIdentifier, []));
      }
    }
  });
}

// src/core/accessor-variable.ts
function accessorVariable(path, accessorIdentifier, callee, replacement) {
  const readIdentifier = path.scope.generateUidIdentifier(accessorIdentifier.name);
  derefMemo(
    path,
    accessorIdentifier,
    readIdentifier
  );
  return t7.variableDeclarator(
    readIdentifier,
    t7.callExpression(callee, replacement)
  );
}

// src/core/deferred-variable.ts
import * as t8 from "@babel/types";
function deferredVariable(state, path, deferredIdentifier, stateIdentifier = t8.identifier("undefined"), optionsIdentifier = void 0) {
  const normalIdentifier = t8.arrowFunctionExpression([], stateIdentifier);
  const args = [normalIdentifier];
  if (state.opts.dev) {
    if (optionsIdentifier) {
      args.push(t8.callExpression(
        t8.memberExpression(
          t8.identifier("Object"),
          t8.identifier("assign")
        ),
        [
          t8.objectExpression([
            t8.objectProperty(
              t8.identifier("name"),
              t8.stringLiteral(deferredIdentifier.name)
            )
          ]),
          optionsIdentifier
        ]
      ));
    } else {
      args.push(t8.objectExpression([
        t8.objectProperty(
          t8.identifier("name"),
          t8.stringLiteral(deferredIdentifier.name)
        )
      ]));
    }
  } else if (optionsIdentifier) {
    args.push(optionsIdentifier);
  }
  return accessorVariable(
    path,
    deferredIdentifier,
    getImportIdentifier(state, path, "createDeferred", "solid-js"),
    args
  );
}

// src/core/deref-memo-variable.ts
import * as t9 from "@babel/types";
function derefMemoVariable(path, memoIdentifier, stateIdentifier) {
  const readIdentifier = path.scope.generateUidIdentifier(memoIdentifier.name);
  derefMemo(
    path,
    memoIdentifier,
    readIdentifier
  );
  return t9.variableDeclarator(readIdentifier, stateIdentifier);
}

// src/core/destructure-variable.ts
import * as t11 from "@babel/types";

// src/core/is-static.ts
import * as t10 from "@babel/types";
function isStatic(node) {
  if (t10.isParenthesizedExpression(node) || t10.isTypeCastExpression(node) || t10.isTSAsExpression(node) || t10.isTSSatisfiesExpression(node) || t10.isTSNonNullExpression(node) || t10.isTSTypeAssertion(node) || t10.isTSInstantiationExpression(node)) {
    return isStatic(node.expression);
  }
  if (t10.isUnaryExpression(node) || t10.isUpdateExpression(node) || t10.isSpreadElement(node)) {
    return isStatic(node.argument);
  }
  if (t10.isRestElement(node)) {
    if (t10.isTSParameterProperty(node.argument)) {
      return false;
    }
    return isStatic(node.argument);
  }
  if (t10.isLiteral(node)) {
    if (t10.isTemplateLiteral(node)) {
      return node.expressions.every((expr) => {
        if (t10.isExpression(expr)) {
          return isStatic(expr);
        }
        return false;
      });
    }
    return true;
  }
  if (t10.isIdentifier(node) || t10.isArrowFunctionExpression(node) || t10.isFunctionExpression(node) || t10.isJSXElement(node) || t10.isJSXFragment(node)) {
    return true;
  }
  if (t10.isArrayExpression(node) || t10.isTupleExpression(node)) {
    return node.elements.every((el) => {
      if (el) {
        return isStatic(el);
      }
      return true;
    });
  }
  if (t10.isArrayPattern(node)) {
    return node.elements.every((el) => {
      if (t10.isTSParameterProperty(el)) {
        return false;
      }
      if (el) {
        return isStatic(el);
      }
      return true;
    });
  }
  if (t10.isObjectExpression(node) || t10.isRecordExpression(node)) {
    return node.properties.every((prop) => {
      if (t10.isObjectProperty(prop)) {
        if (t10.isExpression(prop.value) && isStatic(prop.value)) {
          return true;
        }
        if (prop.computed && t10.isExpression(prop.key) && isStatic(prop.key)) {
          return true;
        }
        return false;
      }
      if (t10.isSpreadElement(prop)) {
        return isStatic(prop);
      }
      return true;
    });
  }
  if (t10.isObjectPattern(node)) {
    return node.properties.every((prop) => {
      if (t10.isObjectProperty(prop)) {
        if (!t10.isTSTypeParameter(prop.value) && isStatic(prop.value)) {
          return true;
        }
        if (prop.computed && t10.isExpression(prop.key) && isStatic(prop.key)) {
          return true;
        }
        return false;
      }
      if (t10.isSpreadElement(prop)) {
        return isStatic(prop);
      }
      return true;
    });
  }
  if (t10.isAssignmentExpression(node) || t10.isAssignmentPattern(node)) {
    if (isStatic(node.right)) {
      return true;
    }
    if (!t10.isTSParameterProperty(node.left)) {
      return false;
    }
    return isStatic(node);
  }
  if (t10.isSequenceExpression(node)) {
    return node.expressions.every(isStatic);
  }
  if (t10.isConditionalExpression(node)) {
    return isStatic(node.test) || isStatic(node.consequent) || isStatic(node.alternate);
  }
  if (t10.isBinaryExpression(node)) {
    if (t10.isExpression(node.left)) {
      return isStatic(node.left);
    }
    if (t10.isExpression(node.right)) {
      return isStatic(node.right);
    }
    return false;
  }
  if (t10.isLogicalExpression(node)) {
    return isStatic(node.left) || isStatic(node.right);
  }
  return false;
}

// src/core/destructure-variable.ts
function destructureVariable(state, path, target, pattern, defaultValue) {
  const otherIdentifier = path.scope.generateUidIdentifier("other");
  let declarators = [];
  const properties = [];
  let restIdentifier;
  if (t11.isObjectPattern(pattern)) {
    for (let i = 0, len = pattern.properties.length; i < len; i++) {
      const property = pattern.properties[i];
      if (t11.isObjectProperty(property)) {
        const { value, key } = property;
        if (!property.computed) {
          if (t11.isIdentifier(key)) {
            properties.push(t11.stringLiteral(key.name));
          }
        } else if (t11.isPrivateName(key)) {
          throw unexpectedType(path, "PrivateName", "Expression");
        } else {
          properties.push(key);
        }
        const newIdentifier = path.scope.generateUidIdentifier("prop");
        let access = t11.memberExpression(
          target,
          key,
          property.computed
        );
        let defaultIdentifier;
        if (t11.isAssignmentPattern(value)) {
          defaultIdentifier = path.scope.generateUidIdentifier("def");
          const isStaticValue = isStatic(value.right);
          const defValue = isStaticValue ? value.right : t11.callExpression(
            getImportIdentifier(state, path, "createMemo", "solid-js"),
            [t11.arrowFunctionExpression([], value.right)]
          );
          declarators.push(
            t11.variableDeclarator(
              defaultIdentifier,
              defValue
            )
          );
          const valueIdentifier = path.scope.generateUidIdentifier("value");
          access = t11.blockStatement([
            t11.variableDeclaration(
              "const",
              [
                t11.variableDeclarator(
                  valueIdentifier,
                  access
                )
              ]
            ),
            t11.returnStatement(
              t11.conditionalExpression(
                t11.binaryExpression(
                  "===",
                  valueIdentifier,
                  t11.identifier("undefined")
                ),
                isStaticValue ? defaultIdentifier : t11.callExpression(defaultIdentifier, []),
                valueIdentifier
              )
            )
          ]);
        }
        declarators.push(
          t11.variableDeclarator(
            newIdentifier,
            t11.arrowFunctionExpression(
              [],
              access
            )
          )
        );
        if (t11.isObjectPattern(value) || t11.isArrayPattern(value)) {
          declarators = [...declarators, ...destructureVariable(
            state,
            path,
            t11.callExpression(newIdentifier, []),
            value
          )];
        } else if (t11.isIdentifier(value)) {
          derefMemo(
            path,
            value,
            newIdentifier
          );
        } else if (t11.isAssignmentPattern(value)) {
          if (t11.isIdentifier(value.left)) {
            derefMemo(
              path,
              value.left,
              newIdentifier
            );
          } else if (t11.isArrayPattern(value.left) || t11.isObjectPattern(value.left)) {
            declarators = [...declarators, ...destructureVariable(
              state,
              path,
              t11.callExpression(newIdentifier, []),
              value.left,
              defaultIdentifier
            )];
          }
        } else {
          throw unexpectedType(path, value.type, "Identifier | ObjectPattern | ArrayPattern");
        }
      } else {
        const trueIdentifier = unwrapNode(property.argument, t11.isIdentifier);
        if (trueIdentifier) {
          restIdentifier = trueIdentifier;
        }
      }
    }
  } else {
    for (let i = 0, len = pattern.elements.length; i < len; i++) {
      const property = pattern.elements[i];
      if (property) {
        const keyExpr = t11.numericLiteral(i);
        const newIdentifier = path.scope.generateUidIdentifier("prop");
        let access = t11.memberExpression(
          target,
          keyExpr,
          true
        );
        let defaultIdentifier;
        if (t11.isAssignmentPattern(property)) {
          defaultIdentifier = path.scope.generateUidIdentifier("def");
          const isStaticValue = isStatic(property.right);
          const defValue = isStaticValue ? property.right : t11.callExpression(
            getImportIdentifier(state, path, "createMemo", "solid-js"),
            [t11.arrowFunctionExpression([], property.right)]
          );
          declarators.push(
            t11.variableDeclarator(
              defaultIdentifier,
              defValue
            )
          );
          const valueIdentifier = path.scope.generateUidIdentifier("value");
          access = t11.blockStatement([
            t11.variableDeclaration(
              "const",
              [
                t11.variableDeclarator(
                  valueIdentifier,
                  access
                )
              ]
            ),
            t11.returnStatement(
              t11.conditionalExpression(
                t11.binaryExpression(
                  "===",
                  valueIdentifier,
                  t11.identifier("undefined")
                ),
                isStaticValue ? defaultIdentifier : t11.callExpression(defaultIdentifier, []),
                valueIdentifier
              )
            )
          ]);
        }
        declarators.push(
          t11.variableDeclarator(
            newIdentifier,
            t11.arrowFunctionExpression(
              [],
              access
            )
          )
        );
        properties.push(keyExpr);
        if (t11.isIdentifier(property)) {
          derefMemo(
            path,
            property,
            newIdentifier
          );
        } else if (t11.isAssignmentPattern(property)) {
          if (t11.isIdentifier(property.left)) {
            derefMemo(
              path,
              property.left,
              newIdentifier
            );
          } else if (t11.isArrayPattern(property.left) || t11.isObjectPattern(property.left)) {
            declarators = [...declarators, ...destructureVariable(
              state,
              path,
              t11.callExpression(newIdentifier, []),
              property.left,
              defaultIdentifier
            )];
          }
        } else if (t11.isArrayPattern(property) || t11.isObjectPattern(property)) {
          declarators = [...declarators, ...destructureVariable(
            state,
            path,
            t11.callExpression(newIdentifier, []),
            property
          )];
        } else if (t11.isRestElement(property)) {
          const trueIdentifier = unwrapNode(property.argument, t11.isIdentifier);
          if (trueIdentifier) {
            restIdentifier = trueIdentifier;
          }
        }
      }
    }
  }
  const expr = t11.variableDeclarator(
    otherIdentifier,
    properties.length ? t11.memberExpression(
      t11.callExpression(
        getImportIdentifier(state, path, "splitProps", "solid-js"),
        [
          defaultValue != null ? t11.callExpression(
            getImportIdentifier(state, path, "mergeProps", "solid-js"),
            [target, defaultValue]
          ) : target,
          t11.arrayExpression(properties)
        ]
      ),
      t11.numericLiteral(1),
      true
    ) : target
  );
  declarators.push(expr);
  if (restIdentifier) {
    const targetName = restIdentifier.name;
    const rest = restIdentifier;
    path.scope.path.traverse({
      ObjectProperty(p) {
        if (!(p.scope !== path.scope && p.scope.hasOwnBinding(targetName)) && p.node.shorthand && t11.isIdentifier(p.node.key) && p.node.key.name === targetName && t11.isIdentifier(p.node.value) && p.node.value.name === targetName) {
          p.replaceWith(
            t11.objectProperty(
              rest,
              otherIdentifier
            )
          );
        }
      },
      Expression(p) {
        if (t11.isIdentifier(p.node) && p.node.name === targetName && !(p.scope !== path.scope && p.scope.hasOwnBinding(targetName))) {
          p.replaceWith(otherIdentifier);
        }
      }
    });
  }
  return declarators;
}

// src/core/memo-variable.ts
import * as t12 from "@babel/types";
function memoVariable(state, path, memoIdentifier, stateIdentifier, optionsIdentifier) {
  const normalIdentifier = t12.arrowFunctionExpression([], stateIdentifier);
  const exprs = [normalIdentifier];
  if (state.opts.dev) {
    exprs.push(t12.identifier("undefined"));
    if (optionsIdentifier) {
      exprs.push(t12.callExpression(
        t12.memberExpression(
          t12.identifier("Object"),
          t12.identifier("assign")
        ),
        [
          t12.objectExpression([
            t12.objectProperty(
              t12.identifier("name"),
              t12.stringLiteral(memoIdentifier.name)
            )
          ]),
          optionsIdentifier
        ]
      ));
    } else {
      exprs.push(t12.objectExpression([
        t12.objectProperty(
          t12.identifier("name"),
          t12.stringLiteral(memoIdentifier.name)
        )
      ]));
    }
  } else if (optionsIdentifier) {
    exprs.push(t12.identifier("undefined"));
    exprs.push(optionsIdentifier);
  }
  return accessorVariable(
    path,
    memoIdentifier,
    getImportIdentifier(state, path, "createMemo", "solid-js"),
    exprs
  );
}

// src/core/signal-variable.ts
import * as t13 from "@babel/types";
function signalVariable(state, path, signalIdentifier, stateIdentifier, optionsIdentifier) {
  const readIdentifier = path.scope.generateUidIdentifier(signalIdentifier.name);
  const writeIdentifier = path.scope.generateUidIdentifier(`set${signalIdentifier.name}`);
  const callee = getImportIdentifier(state, path, "createSignal", "solid-js");
  const args = [stateIdentifier];
  if (state.opts.dev) {
    const nameOption = t13.objectExpression([
      t13.objectProperty(
        t13.identifier("name"),
        t13.stringLiteral(signalIdentifier.name)
      )
    ]);
    if (optionsIdentifier) {
      args.push(
        t13.callExpression(
          t13.memberExpression(
            t13.identifier("Object"),
            t13.identifier("assign")
          ),
          [
            nameOption,
            optionsIdentifier
          ]
        )
      );
    } else {
      args.push(nameOption);
    }
  } else if (optionsIdentifier) {
    args.push(optionsIdentifier);
  }
  derefSignal(
    path,
    signalIdentifier,
    readIdentifier,
    writeIdentifier
  );
  return t13.variableDeclarator(
    t13.arrayPattern([
      readIdentifier,
      writeIdentifier
    ]),
    t13.callExpression(callee, args)
  );
}

// src/transform-ctf.ts
var AUTO_ARROW_CTF = {
  $lazy: ["lazy", "solid-js", 1],
  $untrack: ["untrack", "solid-js", 1],
  $batch: ["batch", "solid-js", 1],
  $observable: ["observable", "solid-js", 1],
  $selector: ["createSelector", "solid-js", 1],
  $on: ["on", "solid-js", 3]
};
var AUTO_IMPORT_ALIAS_CTF = {
  $root: ["createRoot", "solid-js"],
  $useContext: ["useContext", "solid-js"],
  $createContext: ["createContext", "solid-js"],
  $uid: ["createUniqueId", "solid-js"],
  $effect: ["createEffect", "solid-js"],
  $computed: ["createComputed", "solid-js"],
  $renderEffect: ["createRenderEffect", "solid-js"],
  $merge: ["mergeProps", "solid-js"],
  $resource: ["createResource", "solid-js"],
  $cleanup: ["onCleanup", "solid-js"],
  $mount: ["onMount", "solid-js"],
  $error: ["onError", "solid-js"],
  $reaction: ["createReaction", "solid-js"],
  $startTransition: ["startTransition", "solid-js"],
  $useTransition: ["useTransition", "solid-js"],
  $owner: ["getOwner", "solid-js"],
  $runWithOwner: ["runWithOwner", "solid-js"],
  $catchError: ["catchError", "solid-js"],
  // Store API
  $store: ["createStore", "solid-js/store"],
  $mutable: ["createMutable", "solid-js/store"],
  $produce: ["produce", "solid-js/store"],
  $reconcile: ["reconcile", "solid-js/store"],
  $unwrap: ["unwrap", "solid-js/store"]
};
var AUTO_ACCESSOR_CTF = {
  $from: ["from", "solid-js", 1, true],
  $children: ["children", "solid-js", 1, false],
  $mapArray: ["mapArray", "solid-js", 3, false],
  $indexArray: ["indexArray", "solid-js", 3, false]
};
var SIGNAL_CTF = "$signal";
var MEMO_CTF = "$memo";
var DEREF_SIGNAL_CTF = "$derefSignal";
var DEREF_MEMO_CTF = "$derefMemo";
var REACTIVE_CTF = "$";
var DEFERRED_CTF = "$deferred";
var DESTRUCTURE_CTF = "$destructure";
var COMPONENT_CTF = "$component";
var SPECIAL_CTF = /* @__PURE__ */ new Set([
  SIGNAL_CTF,
  MEMO_CTF,
  DEREF_SIGNAL_CTF,
  DEREF_MEMO_CTF,
  REACTIVE_CTF,
  DEFERRED_CTF,
  DESTRUCTURE_CTF,
  COMPONENT_CTF
]);
function transformCTF(state, path) {
  path.traverse({
    CallExpression(p) {
      var _a, _b;
      const trueIdentifier = unwrapNode(p.node.callee, t14.isIdentifier);
      if (trueIdentifier && !p.scope.hasBinding(trueIdentifier.name) && !((_b = (_a = state.opts.disabled) == null ? void 0 : _a.ctf) == null ? void 0 : _b[trueIdentifier.name])) {
        if (trueIdentifier.name in AUTO_ARROW_CTF) {
          const [name, source, limit] = AUTO_ARROW_CTF[trueIdentifier.name];
          const args = p.node.arguments;
          assert(args.length <= limit, unexpectedArgumentLength(p, args.length, limit));
          const [argument, ...rest] = args;
          assert(t14.isExpression(argument), unexpectedType(p, argument.type, "Expression"));
          p.replaceWith(
            t14.callExpression(
              getImportIdentifier(state, p, name, source),
              [
                t14.isArrowFunctionExpression(argument) || t14.isFunctionExpression(argument) ? argument : t14.arrowFunctionExpression([], argument),
                ...rest
              ]
            )
          );
        }
        if (trueIdentifier.name === COMPONENT_CTF) {
          const args = p.node.arguments;
          assert(args.length === 1, unexpectedArgumentLength(p, args.length, 1));
          const argument = args[0];
          assert(t14.isFunctionExpression(argument) || t14.isArrowFunctionExpression(argument), unexpectedType(p, argument.type, "FunctionExpression | ArrowFunctionExpression"));
          if (argument.params.length > 0) {
            const params = argument.params[0];
            if (t14.isObjectPattern(params)) {
              const props = p.scope.generateUidIdentifier("props");
              argument.params[0] = props;
              const declaration = t14.variableDeclaration(
                "const",
                [
                  t14.variableDeclarator(
                    params,
                    t14.callExpression(
                      t14.identifier("$destructure"),
                      [props]
                    )
                  )
                ]
              );
              if (t14.isExpression(argument.body)) {
                argument.body = t14.blockStatement([
                  declaration,
                  t14.returnStatement(argument.body)
                ]);
              } else {
                argument.body.body.unshift(declaration);
              }
            }
          }
          p.replaceWith(argument);
        }
      }
    },
    Expression(p) {
      var _a, _b;
      if (t14.isIdentifier(p.node) && !p.scope.hasBinding(p.node.name)) {
        if (p.node.name in AUTO_IMPORT_ALIAS_CTF && !((_b = (_a = state.opts.disabled) == null ? void 0 : _a.ctf) == null ? void 0 : _b[p.node.name])) {
          const [name, source] = AUTO_IMPORT_ALIAS_CTF[p.node.name];
          p.replaceWith(getImportIdentifier(state, p, name, source));
        }
      }
    },
    VariableDeclarator(p) {
      var _a, _b;
      const { id, init } = p.node;
      if (init) {
        const trueCallExpr = unwrapNode(init, t14.isCallExpression);
        if (trueCallExpr) {
          const trueCallee = unwrapNode(trueCallExpr.callee, t14.isIdentifier);
          if (trueCallee && !p.scope.hasBinding(trueCallee.name) && (SPECIAL_CTF.has(trueCallee.name) || trueCallee.name in AUTO_ACCESSOR_CTF) && !((_b = (_a = state.opts.disabled) == null ? void 0 : _a.ctf) == null ? void 0 : _b[trueCallee.name])) {
            if (t14.isIdentifier(id)) {
              if (trueCallee.name in AUTO_ACCESSOR_CTF) {
                const [name, source, limit, arrow] = AUTO_ACCESSOR_CTF[trueCallee.name];
                const args = trueCallExpr.arguments;
                assert(args.length <= limit, unexpectedArgumentLength(p, args.length, limit));
                const [argument, ...rest] = args;
                assert(t14.isExpression(argument), unexpectedType(p, argument.type, "Expression"));
                p.replaceWith(
                  accessorVariable(
                    p,
                    id,
                    getImportIdentifier(state, p, name, source),
                    [
                      arrow ? t14.arrowFunctionExpression([], argument) : argument,
                      ...rest
                    ]
                  )
                );
              } else if (trueCallee.name === SIGNAL_CTF) {
                const args = trueCallExpr.arguments;
                assert(args.length <= 2, unexpectedArgumentLength(p, args.length, 2));
                let argument;
                let options;
                if (trueCallExpr.arguments.length > 0) {
                  const initialState = trueCallExpr.arguments[0];
                  assert(t14.isExpression(initialState), unexpectedType(p, initialState.type, "Expression"));
                  argument = initialState;
                  if (trueCallExpr.arguments.length > 1) {
                    const optionsValue = trueCallExpr.arguments[1];
                    assert(t14.isExpression(optionsValue), unexpectedType(p, optionsValue.type, "Expression"));
                    options = optionsValue;
                  }
                }
                p.replaceWith(
                  signalVariable(
                    state,
                    p,
                    id,
                    argument || t14.identifier("undefined"),
                    options
                  )
                );
              } else if (trueCallee.name === MEMO_CTF || trueCallee.name === REACTIVE_CTF) {
                const args = trueCallExpr.arguments;
                assert(args.length <= 2, unexpectedArgumentLength(p, args.length, 2));
                const argument = args[0];
                assert(t14.isExpression(argument), unexpectedType(p, argument.type, "Expression"));
                let options;
                if (args.length > 1) {
                  const optionsValue = args[1];
                  assert(t14.isExpression(optionsValue), unexpectedType(p, optionsValue.type, "Expression"));
                  options = optionsValue;
                }
                p.replaceWith(
                  memoVariable(
                    state,
                    p,
                    id,
                    argument,
                    options
                  )
                );
              } else if (trueCallee.name === DEREF_SIGNAL_CTF) {
                const args = trueCallExpr.arguments;
                assert(args.length === 1, unexpectedArgumentLength(p, args.length, 1));
                const argument = args[0];
                assert(t14.isExpression(argument), unexpectedType(p, argument.type, "Expression"));
                p.replaceWith(
                  derefSignalVariable(
                    p,
                    id,
                    argument
                  )
                );
              } else if (trueCallee.name === DEREF_MEMO_CTF) {
                const args = trueCallExpr.arguments;
                assert(args.length === 1, unexpectedArgumentLength(p, args.length, 1));
                const argument = args[0];
                assert(t14.isExpression(argument), unexpectedType(p, argument.type, "Expression"));
                p.replaceWith(
                  derefMemoVariable(
                    p,
                    id,
                    argument
                  )
                );
              } else if (trueCallee.name === DEFERRED_CTF) {
                const args = trueCallExpr.arguments;
                assert(args.length <= 2, unexpectedArgumentLength(p, args.length, 2));
                let argument;
                let options;
                if (args.length > 0) {
                  const initialState = args[0];
                  assert(t14.isExpression(initialState), unexpectedType(p, initialState.type, "Expression"));
                  argument = initialState;
                  if (args.length > 1) {
                    const optionsValue = args[1];
                    assert(t14.isExpression(optionsValue), unexpectedType(p, optionsValue.type, "Expression"));
                    options = optionsValue;
                  }
                }
                p.replaceWith(
                  deferredVariable(
                    state,
                    p,
                    id,
                    argument,
                    options
                  )
                );
              }
            } else if (trueCallee.name === DESTRUCTURE_CTF) {
              const args = trueCallExpr.arguments;
              assert(args.length === 1, unexpectedArgumentLength(p, args.length, 1));
              const argument = args[0];
              assert(t14.isExpression(argument), unexpectedType(p, argument.type, "Expression"));
              assert(t14.isObjectPattern(id) || t14.isArrayPattern(id), unexpectedType(p, id.type, "ArrayPattern | ObjectPattern"));
              p.replaceWithMultiple(
                destructureVariable(
                  state,
                  p,
                  argument,
                  id
                )
              );
            }
          }
        }
      }
    },
    ExpressionStatement(p) {
      var _a, _b;
      const trueCallExpr = unwrapNode(p.node.expression, t14.isCallExpression);
      if (trueCallExpr) {
        const trueCallee = unwrapNode(trueCallExpr.callee, t14.isIdentifier);
        if (trueCallee && !p.scope.hasBinding(trueCallee.name) && !((_b = (_a = state.opts.disabled) == null ? void 0 : _a.ctf) == null ? void 0 : _b[trueCallee.name])) {
          if (trueCallee.name === REACTIVE_CTF) {
            const args = trueCallExpr.arguments;
            assert(args.length === 1, unexpectedArgumentLength(p, args.length, 1));
            const argument = args[0];
            assert(t14.isExpression(argument), unexpectedType(p, argument.type, "Expression"));
            p.replaceWith(
              t14.expressionStatement(
                t14.callExpression(
                  getImportIdentifier(state, p, "createEffect", "solid-js"),
                  [
                    t14.arrowFunctionExpression([], argument)
                  ]
                )
              )
            );
          }
        }
      }
    }
  });
}

// src/transform-label.ts
import * as t15 from "@babel/types";
var REACTIVE_LABEL = "$";
var SPECIAL_LABELS = /* @__PURE__ */ new Set([
  REACTIVE_LABEL
]);
var VARIABLE_LABEL = {
  signal: true,
  memo: true,
  deferred: true,
  destructure: true,
  children: true
};
var CALLBACK_LABEL = {
  effect: ["createEffect", "solid-js", true],
  computed: ["createComputed", "solid-js", true],
  renderEffect: ["createRenderEffect", "solid-js", true],
  mount: ["onMount", "solid-js", false],
  cleanup: ["onCleanup", "solid-js", false],
  error: ["onError", "solid-js", false],
  root: ["createRoot", "solid-js", false],
  untrack: ["untrack", "solid-js", false],
  batch: ["untrack", "solid-js", false],
  transition: ["startTransition", "solid-js", false]
};
function transformLabels(state, path) {
  path.traverse({
    LabeledStatement(p) {
      var _a, _b, _c, _d, _e, _f;
      const labelName = p.node.label.name;
      let { body } = p.node;
      if ((SPECIAL_LABELS.has(labelName) || labelName in VARIABLE_LABEL || labelName in CALLBACK_LABEL) && !((_b = (_a = state.opts.disabled) == null ? void 0 : _a.label) == null ? void 0 : _b[labelName])) {
        if (labelName === REACTIVE_LABEL) {
          let target;
          if (t15.isExpressionStatement(body)) {
            target = body.expression;
          } else if (t15.isBlockStatement(body)) {
            target = body;
          } else {
            throw unexpectedType(p, body.type, "ExpressionStatement | BlockStatement");
          }
          p.replaceWith(
            t15.callExpression(
              getImportIdentifier(state, p, "createEffect", "solid-js"),
              [
                t15.arrowFunctionExpression(
                  [],
                  target
                )
              ]
            )
          );
        }
        if (labelName in VARIABLE_LABEL) {
          assert(t15.isVariableDeclaration(body), unexpectedType(p, p.node.type, "VariableDeclaration"));
          let declarators = [];
          for (let i = 0, len = body.declarations.length; i < len; i++) {
            const declarator = body.declarations[i];
            switch (labelName) {
              case "signal":
                if (t15.isIdentifier(declarator.id)) {
                  declarators.push(
                    signalVariable(
                      state,
                      p,
                      declarator.id,
                      (_c = declarator.init) != null ? _c : t15.identifier("undefined")
                    )
                  );
                }
                break;
              case "memo":
                if (t15.isIdentifier(declarator.id)) {
                  declarators.push(
                    memoVariable(
                      state,
                      p,
                      declarator.id,
                      (_d = declarator.init) != null ? _d : t15.identifier("undefined")
                    )
                  );
                }
                break;
              case "deferred":
                if (t15.isIdentifier(declarator.id)) {
                  declarators.push(
                    deferredVariable(
                      state,
                      p,
                      declarator.id,
                      (_e = declarator.init) != null ? _e : t15.identifier("undefined")
                    )
                  );
                }
                break;
              case "destructure":
                if ((t15.isObjectPattern(declarator.id) || t15.isArrayPattern(declarator.id)) && declarator.init) {
                  declarators = [...declarators, ...destructureVariable(
                    state,
                    p,
                    declarator.init,
                    declarator.id
                  )];
                }
                break;
              case "children":
                if (t15.isIdentifier(declarator.id)) {
                  declarators.push(
                    accessorVariable(
                      p,
                      declarator.id,
                      getImportIdentifier(state, p, "children", "solid-js"),
                      [
                        t15.arrowFunctionExpression(
                          [],
                          (_f = declarator.init) != null ? _f : t15.identifier("undefined")
                        )
                      ]
                    )
                  );
                }
                break;
              default:
                break;
            }
          }
          p.replaceWith(
            t15.variableDeclaration(
              "const",
              declarators
            )
          );
        }
        if (labelName in CALLBACK_LABEL) {
          const [name, source, named] = CALLBACK_LABEL[labelName];
          let nameOption;
          let callback;
          if (named && t15.isLabeledStatement(body)) {
            nameOption = body.label.name;
            body = body.body;
          }
          if (t15.isBlockStatement(body)) {
            callback = t15.arrowFunctionExpression(
              [],
              body
            );
          } else if (t15.isExpressionStatement(body)) {
            callback = body.expression;
          } else {
            throw unexpectedType(p, body.type, "BlockStatement | ExpressionStatement");
          }
          const args = [callback];
          if (named && nameOption) {
            args.push(
              t15.identifier("undefined"),
              t15.objectExpression([
                t15.objectProperty(
                  t15.identifier("name"),
                  t15.stringLiteral(nameOption)
                )
              ])
            );
          }
          p.replaceWith(
            t15.callExpression(
              getImportIdentifier(state, p, name, source),
              args
            )
          );
        }
      }
    }
  });
}

// src/transform-comment.ts
import * as t16 from "@babel/types";
var VARIABLE_LABEL2 = {
  "@signal": true,
  "@memo": true,
  "@deferred": true,
  "@destructure": true,
  "@children": true
};
var CALLBACK_LABEL2 = {
  "@effect": ["createEffect", "solid-js", true],
  "@computed": ["createComputed", "solid-js", true],
  "@renderEffect": ["createRenderEffect", "solid-js", true],
  "@mount": ["onMount", "solid-js", false],
  "@cleanup": ["onCleanup", "solid-js", false],
  "@error": ["onError", "solid-js", false],
  "@root": ["createRoot", "solid-js", false],
  "@untrack": ["untrack", "solid-js", false],
  "@batch": ["untrack", "solid-js", false],
  "@transition": ["startTransition", "solid-js", false]
};
function transformComments(state, path) {
  path.traverse({
    VariableDeclaration(p) {
      var _a, _b, _c, _d, _e, _f;
      const comments = p.node.leadingComments;
      if (comments) {
        let preference;
        for (let i = 0, len = comments.length; i < len; i++) {
          const comment = comments[i];
          const value = comment.value.trim();
          if (value in VARIABLE_LABEL2 && !((_b = (_a = state.opts.disabled) == null ? void 0 : _a.pragma) == null ? void 0 : _b[value])) {
            preference = value;
            comment.value = "";
          }
        }
        if (preference) {
          const { declarations } = p.node;
          let declarators = [];
          for (let i = 0, len = declarations.length; i < len; i++) {
            const declarator = declarations[i];
            switch (preference) {
              case "@signal":
                if (t16.isIdentifier(declarator.id)) {
                  declarators.push(
                    signalVariable(
                      state,
                      p,
                      declarator.id,
                      (_c = declarator.init) != null ? _c : t16.identifier("undefined")
                    )
                  );
                }
                break;
              case "@memo":
                if (t16.isIdentifier(declarator.id)) {
                  declarators.push(
                    memoVariable(
                      state,
                      p,
                      declarator.id,
                      (_d = declarator.init) != null ? _d : t16.identifier("undefined")
                    )
                  );
                }
                break;
              case "@deferred":
                if (t16.isIdentifier(declarator.id)) {
                  declarators.push(
                    deferredVariable(
                      state,
                      p,
                      declarator.id,
                      (_e = declarator.init) != null ? _e : t16.identifier("undefined")
                    )
                  );
                }
                break;
              case "@destructure":
                if ((t16.isObjectPattern(declarator.id) || t16.isArrayPattern(declarator.id)) && declarator.init) {
                  declarators = [...declarators, ...destructureVariable(
                    state,
                    p,
                    declarator.init,
                    declarator.id
                  )];
                }
                break;
              case "@children":
                if (t16.isIdentifier(declarator.id)) {
                  declarators.push(
                    accessorVariable(
                      p,
                      declarator.id,
                      getImportIdentifier(state, p, "children", "solid-js"),
                      [
                        t16.arrowFunctionExpression(
                          [],
                          (_f = declarator.init) != null ? _f : t16.identifier("undefined")
                        )
                      ]
                    )
                  );
                }
                break;
              default:
                break;
            }
          }
          p.replaceWith(
            t16.variableDeclaration(
              "const",
              declarators
            )
          );
        }
      }
    },
    BlockStatement(p) {
      var _a, _b;
      if (!t16.isBlockStatement(p.parent)) {
        return;
      }
      const comments = p.node.leadingComments;
      if (comments) {
        let preference;
        let nameOption;
        for (let i = 0, len = comments.length; i < len; i++) {
          const comment = comments[i];
          const value = comment.value.trim();
          if (/^@\w+( .*)?$/.test(value)) {
            const [tag, ...debugName] = value.split(" ");
            if (tag in CALLBACK_LABEL2 && !((_b = (_a = state.opts.disabled) == null ? void 0 : _a.pragma) == null ? void 0 : _b[value])) {
              preference = tag;
              nameOption = debugName.join(" ");
              comment.value = "";
            }
          }
        }
        if (preference) {
          const [name, source, named] = CALLBACK_LABEL2[preference];
          const callback = t16.arrowFunctionExpression(
            [],
            p.node
          );
          const args = [callback];
          if (named && nameOption) {
            args.push(
              t16.identifier("undefined"),
              t16.objectExpression([
                t16.objectProperty(
                  t16.identifier("name"),
                  t16.stringLiteral(nameOption)
                )
              ])
            );
          }
          p.replaceWith(
            t16.callExpression(
              getImportIdentifier(state, p, name, source),
              args
            )
          );
        }
      }
    },
    ExpressionStatement(p) {
      var _a, _b;
      const comments = p.node.leadingComments;
      if (comments) {
        let preference;
        let nameOption;
        for (let i = 0, len = comments.length; i < len; i++) {
          const comment = comments[i];
          const value = comment.value.trim();
          if (/^@\w+( .*)?$/.test(value)) {
            const [tag, ...debugName] = value.split(" ");
            if (tag in CALLBACK_LABEL2 && !((_b = (_a = state.opts.disabled) == null ? void 0 : _a.pragma) == null ? void 0 : _b[value])) {
              preference = tag;
              nameOption = debugName.join(" ");
              comment.value = "";
            }
          }
        }
        if (preference) {
          const [name, source, named] = CALLBACK_LABEL2[preference];
          const callback = t16.arrowFunctionExpression(
            [],
            p.node.expression
          );
          const args = [callback];
          if (named && nameOption) {
            args.push(
              t16.identifier("undefined"),
              t16.objectExpression([
                t16.objectProperty(
                  t16.identifier("name"),
                  t16.stringLiteral(nameOption)
                )
              ])
            );
          }
          p.replaceWith(
            t16.callExpression(
              getImportIdentifier(state, p, name, source),
              args
            )
          );
        }
      }
    }
  });
}

// src/index.ts
function solidLabelsPlugin() {
  return {
    name: "solid-labels",
    pre() {
      this.hooks = /* @__PURE__ */ new Map();
    },
    visitor: {
      Program(path, state) {
        transformComments(state, path);
        transformLabels(state, path);
        transformCTF(state, path);
        transformComponents(state, path);
      }
    }
  };
}
export {
  solidLabelsPlugin as default
};
//# sourceMappingURL=index.mjs.map
